@@ -96,7 +96,7 @@ covcor_design <- function(.data,
   factors <- select(.data,
                     GEN = {{gen}},
                     REP = {{rep}}) %>%
-    to_factor(1:2)
+    as_factor(1:2)
   GEN <- factors$GEN
   REP <- factors$REP
   NREP <- nlevels(REP)
