@@ -37,8 +37,7 @@ make_mat <- function(.data, row, col, value, fun = mean) {
            {{col}},
            {{value}}) %>%
     group_by({{row}}, {{col}}) %>%
-    summarise_if(is.numeric, fun, na.rm = TRUE) %>%
-    ungroup() %>%
+    summarise(across(where(is.numeric), fun, na.rm = TRUE), .groups = "drop") %>%
     pivot_wider(names_from = {{col}}, values_from = {{value}})
   data %<>% column_to_rownames(var = names(data[1]))
   return(data)
