@@ -4,13 +4,15 @@
 #'   analysis. This collection of functions will make it easier to deal with
 #'   them.
 #' @name utils_na_zero
-#' @param .data A data frame or tibble
-#' @param ... Variables to replace \code{NAs} in \code{replace_na()} or zeros in
-#'   \code{replace_zero()}  . If \code{...} is null then all variables with
-#'   \code{NA} or \code{0} will be replaced by the value in \code{replacement}
-#'   argument of such functions. It must be a single variable name or a
-#'   comma-separated list of unquoted variables names. Select helpers are also
-#'   allowed.
+#' @param .data A data frame.
+#' @param ... Variables to fill  \code{NAs} in \code{fill_na()}, replace
+#'   \code{NAs} in \code{replace_na()} or zeros in \code{replace_zero()}. If
+#'   \code{...} is null then all variables in \code{.data} will be evaluated. It
+#'   must be a single variable name or a comma-separated list of unquoted
+#'   variables names. Select helpers are also allowed.
+#' @param direction Direction in which to fill missing values. Currently either
+#'   "down" (the default), "up", "downup" (i.e. first down and then up) or
+#'   "updown" (first up and then down).
 #' @param prop The proportion (percentage) of \code{NA} values to generate in
 #'   \code{.data}.
 #' @param replace Deprecated argument as of 1.8.0. Use \code{replacement} instead.
@@ -21,7 +23,9 @@
 #' @export
 #' @return A data frame with rows or columns with \code{NA} values deleted.
 #' @description
-#' * \code{has_na(), has_zero() }: Check for \code{NAs} and \code{0s} in the
+#' * \code{fill_na()}: Fills \code{NA} in selected columns using the next or
+#' previous entry.
+#' * \code{has_na(), has_zero()}: Check for \code{NAs} and \code{0s} in the
 #' data and return a logical value.
 #' * \code{random_na()}: Generate random \code{NA} values in a two-way table
 #' based on a desired proportion.
@@ -33,19 +37,26 @@
 #' and \code{0s}, respectively.
 #' * \code{select_rows_na(), select_rows_zero()}: Select rows with \code{NAs}
 #' and \code{0s}, respectively.
-#' * \code{replace_na(), replace_zero()} Replace \code{NAs} and \code{0s},
+#' * \code{replace_na(), replace_zero()}: Replace \code{NAs} and \code{0s},
 #' respectively, with a \code{replacement} value.
 #' @md
 #' @author Tiago Olivoto \email{tiagoolivoto@@gmail.com}
 #' @examples
 #' \donttest{
 #' library(metan)
-#' data_naz <- data_g
-#' data_naz[c(1, 5, 10), c(3:5, 10:15)] <- NA
-#' data_naz[c(2, 6, 9), c(6:7, 12:13)] <- 0
+#' data_naz <- iris %>%
+#'               group_by(Species) %>%
+#'               doo(~head(., n = 3)) %>%
+#'               as_character(Species)
+#' data_naz
+#' data_naz[c(2:3, 6, 8), c(1:2, 4, 5)] <- NA
+#' data_naz[c(2, 7, 9), c(2, 3, 4)] <- 0
 #' has_na(data_naz)
 #' has_zero(data_naz)
 #'
+#' # Fill NA values of column GEN
+#' fill_na(data_naz, Species)
+#'
 #' # Remove columns
 #' remove_cols_na(data_naz)
 #' remove_cols_zero(data_naz)
@@ -66,6 +77,19 @@
 
 # Dealing with NAs
 
+fill_na <- function(.data, ..., direction = "down"){
+  if(!direction %in% c("down", "up", "downup", "updown")){
+    stop("Argument 'direction' must be one of 'down', 'up', 'downup', or 'updown'. ")
+  }
+  if(missing(...)){
+    return(fill(.data, everything(), .direction = direction))
+  } else{
+    return(fill(.data, ..., .direction = direction))
+  }
+}
+#' @name utils_na_zero
+#' @export
+#' @importFrom tidyr fill
 has_na <- function(.data){
   any(complete.cases(.data) ==  FALSE)
 }
